#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>
#include <getopt.h>
#include <errno.h>
#include <sys/ioctl.h>

#include "loop.h"

int slurp(const char *filename, unsigned char *buf, size_t bufsize)
{
	ssize_t n;
	int fd = open(filename, O_RDONLY);
	int err = -1;

	if (fd < 0) {
		perror(filename);
		return -1;
	}

	n = read(fd, buf, bufsize);
	if (n < 0) {
		perror("read");
		goto fail;
	}

	if (n < bufsize)
		fprintf(stderr, "Warn: keyfile smaller than expected\n");

	err = 0;
fail:
	close(fd);
	return err;
}

int main(int argc, char **argv)
{
	int loopfd;
	int imagefd;
	struct loop_info64 loopinfo;
	unsigned char key[65*32];
	int err;
	int mode = 3;
	int opt;

	while ((opt = getopt(argc, argv, "m:")) != -1) {
		switch (opt) {
		case 'm':
			mode = atoi(optarg);
			break;
		}
	}

	if (mode < 2 || mode > 3) {
		fprintf(stderr, "Bad mode: %d", mode);
		exit(1);
	}

	if (argc - optind != 3) {
		fprintf(stderr, "usage: %s [-m <mode>] <loopdev> <imgagefile> <keyfile>\n", argv[0]);
		exit(1);
	}

	loopfd = open(argv[optind], O_RDWR);
	if (loopfd < 0) {
		perror(argv[optind]);
		exit(1);
	}

	imagefd = open(argv[optind+1], O_RDWR);
	if (imagefd < 0) {
		perror(argv[optind+1]);
		goto error;
	}

	err = ioctl(loopfd, LOOP_SET_FD, imagefd);
	if (err) {
		perror("loop_set_fd");
		goto error;
	}

	memset(&loopinfo, 0, sizeof(loopinfo));

	loopinfo.lo_encrypt_type = 16; /* AES */
	loopinfo.lo_encrypt_key_size = 16; /* AES128 */

	err = ioctl(loopfd, LOOP_SET_STATUS64, &loopinfo);
	if (err) {
		perror("loop_set_status64\n");
		goto error;
	}

	if (slurp(argv[optind+2], key, sizeof(key)))
		goto error;

	switch (mode) {
	case 2:
		err = ioctl(loopfd, LOOP_MULTI_KEY_SETUP, key);
		break;

	case 3:
		err = ioctl(loopfd, LOOP_MULTI_KEY_SETUP_V3, key);
		break;
	}

	if (err) {
		perror("loop_multi_key_setup");
		goto error;
	}

	err = 0;

error:
	if (err)
		fprintf(stderr, "Aborting due to errors.\n");

	close(loopfd);
	close(imagefd);

	memset(key, 0, sizeof(key));

	exit(err);
}
